/*
 *  MiniAudio.h
 *
 *  A tiny class to hold properties and functions for audio output.

 ***** PIN MAPPING ********************************************
At this time we will use SendOnlySoftwareSerial in order to only need a single pin

D10................Serial Control Out

 */

#ifndef _MINIAUDIO_H
#define _MINIAUDIO_H

#include <Arduino.h>
#include <avr/pgmspace.h>

#define AUDIO_PIN 10
//#define bit9600Delay 84  
//#define bit4800Delay 188 

/************catalex mp3 board constants**************************/
static int8_t Send_buf[8] = {0} ;

#define CMD_PLAY_W_INDEX 0X08
#define CMD_SET_VOLUME 0X06
#define CMD_SEL_DEV 0X09
#define DEV_TF 0X02
#define CMD_STOP_PLAY 0X16
#define CMD_PLAY_W_VOL 0X22

/*********************************************************************/

class MiniAudio
{

  public:
    MiniAudio();
    void Init(); 
	void PlayAmbient();
	void PlayScare();
	void Stop();
	void Report();

  private:
  uint8_t _transmitBitMask;
  volatile uint8_t *_transmitPortRegister;
  uint16_t _tx_delay;
  // private methods
  void tx_pin_write(uint8_t pin_state);
  void write(uint8_t byte);
  // private static method for timing
  static inline void tunedDelay(uint16_t delay);
  void sendCommand(int8_t command, int16_t dat);
};

MiniAudio::MiniAudio()
{
	pinMode(AUDIO_PIN, OUTPUT);
	digitalWrite(AUDIO_PIN, HIGH);
	_transmitBitMask = digitalPinToBitMask(AUDIO_PIN);
	uint8_t port = digitalPinToPort(AUDIO_PIN);
	_transmitPortRegister = portOutputRegister(port);
}

void MiniAudio::Init()
{
	pinMode(AUDIO_PIN,OUTPUT);
	digitalWrite(AUDIO_PIN,LOW);
	delay(500);
	_tx_delay = 238; //9600
	sendCommand(CMD_SEL_DEV, DEV_TF);  //select the TF card  
	delay(500);
	sendCommand(CMD_SET_VOLUME, 0X1E); //volume up full blast
	delay(200);
}
void MiniAudio::PlayAmbient()
{
	//sendCommand(CMD_SET_VOLUME, 0X1E); //volume up full blast
	//delay(100);
	sendCommand(CMD_PLAY_W_INDEX, 0x01); //CMD_PLAY_W_INDEX loops
}
void MiniAudio::PlayScare()
{
	sendCommand(CMD_PLAY_W_VOL, 0x1E02); //CMD_PLAY_W_VOL plays 1x
}
void MiniAudio::Stop()
{
	sendCommand(CMD_STOP_PLAY,0);
}
void MiniAudio::sendCommand(int8_t command, int16_t dat)
{
	delay(20);
	Send_buf[0] = 0x7e; //
	Send_buf[1] = 0xff; //
	Send_buf[2] = 0x06; //
	Send_buf[3] = command; //
	Send_buf[4] = 0x00;//
	Send_buf[5] = (int8_t)(dat >> 8);//datah
	Send_buf[6] = (int8_t)(dat); //datal
	Send_buf[7] = 0xef; //
	for(uint8_t i=0; i<8; i++)//
	{
	write(Send_buf[i]) ;
	}
}

/* static */ 
inline void MiniAudio::tunedDelay(uint16_t delay) { 
  uint8_t tmp=0;

  asm volatile("sbiw    %0, 0x01 \n\t"
    "ldi %1, 0xFF \n\t"
    "cpi %A0, 0xFF \n\t"
    "cpc %B0, %1 \n\t"
    "brne .-10 \n\t"
    : "+r" (delay), "+a" (tmp)
    : "0" (delay)
    );
}


void MiniAudio::tx_pin_write(uint8_t pin_state)
{
  if (pin_state == LOW)
    *_transmitPortRegister &= ~_transmitBitMask;
  else
    *_transmitPortRegister |= _transmitBitMask;
}


void MiniAudio::write(uint8_t b)
{
  if (_tx_delay == 0) return;

  uint8_t oldSREG = SREG;
  cli();  // turn off interrupts for a clean txmit

  // Write the start bit
  tx_pin_write(LOW);
  tunedDelay(_tx_delay);

	for (byte mask = 0x01; mask; mask <<= 1)
	{
		if (b & mask) // choose bit
		tx_pin_write(HIGH); // send 1
		else
		tx_pin_write(LOW); // send 0
    
		tunedDelay(_tx_delay);
	}

	tx_pin_write(HIGH); // restore pin to natural state

  SREG = oldSREG; // turn interrupts back on
  tunedDelay(_tx_delay);

}
#endif // _MINIAUDIO_H
